library HmsWmpAddon;

uses
  Types,
  Messages,
  SysUtils,
  Classes,
  ActiveX,
  Windows,
  Controls,
  Forms,
  Variants,
  HmsAddonsIntfTypes in 'HmsAddonsIntfTypes.pas',
  WMPLib_TLB in 'WMPLib_TLB.pas';

{$R *.res}

type
  THmsAddonList = class(TComponent, IHmsAddonList, IHmsAddonTools)
  private
    FNeedToUninitialize: Boolean;
    FObjectList: TList;
    procedure FreeObjects;
  protected
    function AddObject(aObject: TComponent): TComponent;

    property ObjectList: TList read FObjectList;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  { IHmsAddonTools }
    function Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
    function Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
  { IHmsAddonList }
    function GetCount(var aCount: Integer): HResult; safecall;
    function GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
    function GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
    function CanUnloadNow: HResult; safecall;
  end;

  THmsMainFormControl = class(TComponent, IHmsMainFormControl)
  private
    FControl: TWinControl;
    FPlayer: TWindowsMediaPlayer;
  protected
    function CreateControl(aParent: THandle; const aReserved: IUnknown; var aControl: THandle): HResult; safecall;
    function DestroyControl(aControl: THandle): HResult; safecall;
    function GetCaption(var aCaption: OleVariant): HResult; safecall;

    function ExecuteAction(const aActionName: OleVariant; const aActionParameters: OleVariant): HResult; reintroduce; safecall;

    function GetActionCount(var aCount: Integer): HResult; safecall;
    function GetActionInfo(aIndex: Integer; var aActionName, aActionTitle, aActionDescription: OleVariant): HResult; safecall;

    function SetFocus: HResult; safecall;
  public
    destructor Destroy; override;
  end;

{ THmsAddonList }

type
  THmsAddonInfo = record
    ClassID: TGUID;
    InterfaceID: TGUID;
    Title: string;
    Description: string;
    RequiredVersion: string;
    CheckedOnVersion: string;
  end;

const
  AddonInfoCount = 1;

  CLASS_HmsMainFormControl = '{6BD2B590-4CE5-488E-8B2E-7A71BBA07473}';

  csAddonTitle = 'Windows Media Player';

  AddonInfoList: array[0..AddonInfoCount - 1] of THmsAddonInfo =
    ((ClassID : CLASS_HmsMainFormControl;
      InterfaceID: IID_HmsMainFormControl;
      Title : csAddonTitle;
      Description: ' "Windows Media Player"';
      RequiredVersion : '2.04';
      CheckedOnVersion: '2.04'));

{ THmsAddonList }

constructor THmsAddonList.Create;
begin
  FNeedToUninitialize := Succeeded(CoInitialize(nil));
  inherited Create(nil);
  FObjectList := TList.Create;
end;

destructor THmsAddonList.Destroy;
begin
  FreeObjects;
  inherited Destroy;
end;

function THmsAddonList.AddObject(aObject: TComponent): TComponent;
begin
  FObjectList.Add(aObject);
  aObject.FreeNotification(Self);
  Result := aObject
end;

function THmsAddonList.GetCount(var aCount: Integer): HResult; safecall;
begin
  aCount := AddonInfoCount;
  Result := S_OK
end;

function THmsAddonList.GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
begin
  if (aIndex >= 0) and (aIndex < AddonInfoCount) then begin
    with AddonInfoList[aIndex] do begin
      aClassID := ClassID;
      aInterfaceID := InterfaceID;
      aTitle := Title;
      aDescription := Description;
      aRequiredVersion := RequiredVersion;
      aCheckedOnVersion := CheckedOnVersion
    end;
    Result := S_OK
  end else
    Result := E_UNEXPECTED
end;

function THmsAddonList.GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
begin
  Result := E_NOINTERFACE;
  if IsEqualGUID(aInterfaceID, IHmsMainFormControl) then begin
    aObject := AddObject(THmsMainFormControl.Create(nil)) as IHmsMainFormControl;
    Result := S_OK
  end
end;

procedure THmsAddonList.FreeObjects;
var
  i: Integer;
begin
  for i := FObjectList.Count - 1 downto 0 do try
    TObject(FObjectList[i]).Free;
  except end;
  FObjectList.Clear
end;

function THmsAddonList.CanUnloadNow: HResult; safecall;
begin
  FreeObjects;
  if FNeedToUninitialize then
    CoUninitialize;
  Result := S_OK
end;

procedure THmsAddonList.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    FObjectList.Remove(AComponent)
end;

function THmsAddonList.Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
begin
  aReload := MessageBox(aOwner, '  ?', '  ', MB_ICONQUESTION + MB_YESNO) = IDYES;
  Result := S_OK
end;

function THmsAddonList.Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
begin
  aFlags := ufIsInfoMessage;
  aResult := '  ';
  Result := S_OK
end;

{ THmsMainFormControl }

destructor THmsMainFormControl.Destroy;
begin
  FreeAndNil(FControl);
  inherited Destroy;
end;

{.$DEFINE OnPosRectChange_Virtual}

type
  THmsWindowsMediaPlayer = class(TWindowsMediaPlayer)
  protected
  {$IFDEF OnPosRectChange_Virtual}
    function OnPosRectChange(const rcPosRect: TRect): HResult; override;
  {$ENDIF}
  end;

{ THmsWindowsMediaPlayer }

{$IFDEF OnPosRectChange_Virtual}

function THmsWindowsMediaPlayer.OnPosRectChange(const rcPosRect: TRect): HResult;
begin
  Result := inherited OnPosRectChange(Rect(0, 0, Width, Height))
end;

{$ENDIF}

function THmsMainFormControl.CreateControl(aParent: THandle; const aReserved: IUnknown; var aControl: THandle): HResult; safecall;
var
  ParentClientRect: TRect;
begin
  if FControl = nil then begin
    FControl := TWinControl.CreateParented(aParent);
    GetClientRect(aParent, ParentClientRect);
    FControl.BoundsRect := ParentClientRect;
    FPlayer := THmsWindowsMediaPlayer.Create(FControl);
    FPlayer.Parent := FControl;
    FPlayer.Align := alClient;
  end;
  aControl := FControl.Handle;
  Result := S_OK
end;

function THmsMainFormControl.DestroyControl(aControl: THandle): HResult; safecall;
begin
  FreeAndNil(FControl);
  Result := S_OK
end;

function THmsMainFormControl.GetCaption(var aCaption: OleVariant): HResult; safecall;
begin
  aCaption := csAddonTitle;
  Result := S_OK
end;

function THmsMainFormControl.SetFocus: HResult;
begin
  if FPlayer <> nil then
    Windows.SetFocus(FPlayer.Handle);
  Result := S_OK
end;

function THmsMainFormControl.ExecuteAction(const aActionName: OleVariant; const aActionParameters: OleVariant): HResult; safecall;
var
  sActionName: string;
begin
  Result := E_FAIL;
  try
    if FPlayer <> nil then begin
      sActionName := VarToStr(aActionName);
      if SameText(sActionName, 'open') then begin
        if VarIsStr(aActionParameters) then begin
          FPlayer.currentMedia := FPlayer.newMedia(VarToStr(aActionParameters));
          Result := S_OK
        end
      end
    end
  except end
end;

function THmsMainFormControl.GetActionCount(var aCount: Integer): HResult; safecall;
begin
  Result := E_NOTIMPL
end;

function THmsMainFormControl.GetActionInfo(aIndex: Integer; var aActionName, aActionTitle, aActionDescription: OleVariant): HResult; safecall;
begin
  Result := E_NOTIMPL
end;

function HmsGetAddonList(out aAddonList: IHmsAddonList): HResult; stdcall;
begin
  aAddonList := THmsAddonList.Create;
  Result := S_OK
end;

exports
  HmsGetAddonList;

end.
